@@ -107,49 +107,34 @@ ngx_http_lua_inject_string_api(lua_State *L)
 static int
 ngx_http_lua_ngx_escape_uri(lua_State *L)
 {
-    ngx_http_request_t      *r;
     size_t                   len, dlen;
     uintptr_t                escape;
     u_char                  *src, *dst;
 
-    lua_pushlightuserdata(L, &ngx_http_lua_request_key);
-    lua_rawget(L, LUA_GLOBALSINDEX);
-    r = lua_touserdata(L, -1);
-    lua_pop(L, 1);
-
-    if (r == NULL) {
-        return luaL_error(L, "no request object found");
-    }
-
     if (lua_gettop(L) != 1) {
         return luaL_error(L, "expecting one argument");
     }
 
+    if (lua_isnil(L, 1)) {
+        lua_pushliteral(L, "");
+        return 1;
+    }
+
     src = (u_char *) luaL_checklstring(L, 1, &len);
 
     if (len == 0) {
-        lua_pushlstring(L, NULL, 0);
         return 1;
     }
 
     escape = 2 * ngx_http_lua_escape_uri(NULL, src, len, NGX_ESCAPE_URI);
 
-    dlen = escape + len;
-
-    dst = ngx_palloc(r->pool, dlen);
-    if (dst == NULL) {
-        return luaL_error(L, "memory allocation error");
-    }
-
-    if (escape == 0) {
-        ngx_memcpy(dst, src, len);
-
-    } else {
+    if (escape) {
+        dlen = escape + len;
+        dst = lua_newuserdata(L, dlen);
         ngx_http_lua_escape_uri(dst, src, len, NGX_ESCAPE_URI);
+        lua_pushlstring(L, (char *) dst, dlen);
     }
 
-    lua_pushlstring(L, (char *) dst, dlen);
-
     return 1;
 }
 
@@ -157,33 +142,25 @@ ngx_http_lua_ngx_escape_uri(lua_State *L)
 static int
 ngx_http_lua_ngx_unescape_uri(lua_State *L)
 {
-    ngx_http_request_t      *r;
     size_t                   len, dlen;
     u_char                  *p;
     u_char                  *src, *dst;
 
-    lua_pushlightuserdata(L, &ngx_http_lua_request_key);
-    lua_rawget(L, LUA_GLOBALSINDEX);
-    r = lua_touserdata(L, -1);
-    lua_pop(L, 1);
-
-    if (r == NULL) {
-        return luaL_error(L, "no request object found");
-    }
-
     if (lua_gettop(L) != 1) {
         return luaL_error(L, "expecting one argument");
     }
 
+    if (lua_isnil(L, 1)) {
+        lua_pushliteral(L, "");
+        return 1;
+    }
+
     src = (u_char *) luaL_checklstring(L, 1, &len);
 
     /* the unescaped string can only be smaller */
     dlen = len;
 
-    p = ngx_palloc(r->pool, dlen);
-    if (p == NULL) {
-        return luaL_error(L, "memory allocation error");
-    }
+    p = lua_newuserdata(L, dlen);
 
     dst = p;
 
@@ -198,20 +175,10 @@ ngx_http_lua_ngx_unescape_uri(lua_State *L)
 static int
 ngx_http_lua_ngx_quote_sql_str(lua_State *L)
 {
-    ngx_http_request_t      *r;
     size_t                   len, dlen, escape;
     u_char                  *p;
     u_char                  *src, *dst;
 
-    lua_pushlightuserdata(L, &ngx_http_lua_request_key);
-    lua_rawget(L, LUA_GLOBALSINDEX);
-    r = lua_touserdata(L, -1);
-    lua_pop(L, 1);
-
-    if (r == NULL) {
-        return luaL_error(L, "no request object found");
-    }
-
     if (lua_gettop(L) != 1) {
         return luaL_error(L, "expecting one argument");
     }
@@ -229,10 +196,7 @@ ngx_http_lua_ngx_quote_sql_str(lua_State *L)
 
     dlen = sizeof("''") - 1 + len + escape;
 
-    p = ngx_palloc(r->pool, dlen);
-    if (p == NULL) {
-        return luaL_error(L, "out of memory");
-    }
+    p = lua_newuserdata(L, dlen);
 
     dst = p;
 
@@ -248,8 +212,6 @@ ngx_http_lua_ngx_quote_sql_str(lua_State *L)
     *p++ = '\'';
 
     if (p != dst + dlen) {
-        ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
-                      "ngx.quote_sql_str: buffer error");
         return NGX_ERROR;
     }
 
@@ -260,8 +222,7 @@ ngx_http_lua_ngx_quote_sql_str(lua_State *L)
 
 
 static uintptr_t
-ngx_http_lua_ngx_escape_sql_str(u_char *dst, u_char *src,
-        size_t size)
+ngx_http_lua_ngx_escape_sql_str(u_char *dst, u_char *src, size_t size)
 {
     ngx_uint_t               n;
 
@@ -273,12 +234,15 @@ ngx_http_lua_ngx_escape_sql_str(u_char *dst, u_char *src,
              * is always 1 */
             if ((*src & 0x80) == 0) {
                 switch (*src) {
-                    case '\r':
+                    case '\0':
+                    case '\b':
                     case '\n':
+                    case '\r':
+                    case '\t':
+                    case 26:  /* \z */
                     case '\\':
                     case '\'':
                     case '"':
-                    case '\032':
                         n++;
                         break;
                     default:
@@ -295,9 +259,14 @@ ngx_http_lua_ngx_escape_sql_str(u_char *dst, u_char *src,
     while (size) {
         if ((*src & 0x80) == 0) {
             switch (*src) {
-                case '\r':
+                case '\0':
                     *dst++ = '\\';
-                    *dst++ = 'r';
+                    *dst++ = '0';
+                    break;
+
+                case '\b':
+                    *dst++ = '\\';
+                    *dst++ = 'b';
                     break;
 
                 case '\n':
@@ -305,6 +274,21 @@ ngx_http_lua_ngx_escape_sql_str(u_char *dst, u_char *src,
                     *dst++ = 'n';
                     break;
 
+                case '\r':
+                    *dst++ = '\\';
+                    *dst++ = 'r';
+                    break;
+
+                case '\t':
+                    *dst++ = '\\';
+                    *dst++ = 't';
+                    break;
+
+                case 26:
+                    *dst++ = '\\';
+                    *dst++ = 'z';
+                    break;
+
                 case '\\':
                     *dst++ = '\\';
                     *dst++ = '\\';
@@ -320,11 +304,6 @@ ngx_http_lua_ngx_escape_sql_str(u_char *dst, u_char *src,
                     *dst++ = '"';
                     break;
 
-                case '\032':
-                    *dst++ = '\\';
-                    *dst++ = *src;
-                    break;
-
                 default:
                     *dst++ = *src;
                     break;
@@ -354,9 +333,9 @@ ngx_http_lua_ngx_md5(lua_State *L)
         return luaL_error(L, "expecting one argument");
     }
 
-    if (strcmp(luaL_typename(L, 1), (char *) "nil") == 0) {
-        src     = (u_char *) "";
-        slen    = 0;
+    if (lua_isnil(L, 1)) {
+        src = (u_char *) "";
+        slen = 0;
 
     } else {
         src = (u_char *) luaL_checklstring(L, 1, &slen);
@@ -445,36 +424,21 @@ ngx_http_lua_ngx_sha1_bin(lua_State *L)
 static int
 ngx_http_lua_ngx_decode_base64(lua_State *L)
 {
-    ngx_http_request_t      *r;
     ngx_str_t                p, src;
 
-    lua_pushlightuserdata(L, &ngx_http_lua_request_key);
-    lua_rawget(L, LUA_GLOBALSINDEX);
-    r = lua_touserdata(L, -1);
-    lua_pop(L, 1);
-
-    if (r == NULL) {
-        return luaL_error(L, "no request object found");
-    }
-
     if (lua_gettop(L) != 1) {
         return luaL_error(L, "expecting one argument");
     }
 
-    if (strcmp(luaL_typename(L, 1), (char *) "nil") == 0) {
-        src.data     = (u_char *) "";
-        src.len      = 0;
-
-    } else {
-        src.data = (u_char *) luaL_checklstring(L, 1, &src.len);
+    if (lua_type(L, 1) != LUA_TSTRING) {
+        return luaL_error(L, "string argument only");
     }
 
+    src.data = (u_char *) luaL_checklstring(L, 1, &src.len);
+
     p.len = ngx_base64_decoded_length(src.len);
 
-    p.data = ngx_palloc(r->pool, p.len);
-    if (p.data == NULL) {
-        return NGX_ERROR;
-    }
+    p.data = lua_newuserdata(L, p.len);
 
     if (ngx_decode_base64(&p, &src) == NGX_OK) {
         lua_pushlstring(L, (char *) p.data, p.len);
@@ -483,8 +447,6 @@ ngx_http_lua_ngx_decode_base64(lua_State *L)
         lua_pushnil(L);
     }
 
-    ngx_pfree(r->pool, p.data);
-
     return 1;
 }
 
@@ -492,25 +454,15 @@ ngx_http_lua_ngx_decode_base64(lua_State *L)
 static int
 ngx_http_lua_ngx_encode_base64(lua_State *L)
 {
-    ngx_http_request_t      *r;
     ngx_str_t                p, src;
 
-    lua_pushlightuserdata(L, &ngx_http_lua_request_key);
-    lua_rawget(L, LUA_GLOBALSINDEX);
-    r = lua_touserdata(L, -1);
-    lua_pop(L, 1);
-
-    if (r == NULL) {
-        return luaL_error(L, "no request object found");
-    }
-
     if (lua_gettop(L) != 1) {
         return luaL_error(L, "expecting one argument");
     }
 
-    if (strcmp(luaL_typename(L, 1), (char *) "nil") == 0) {
-        src.data     = (u_char *) "";
-        src.len      = 0;
+    if (lua_isnil(L, 1)) {
+        src.data = (u_char *) "";
+        src.len = 0;
 
     } else {
         src.data = (u_char *) luaL_checklstring(L, 1, &src.len);
@@ -518,17 +470,12 @@ ngx_http_lua_ngx_encode_base64(lua_State *L)
 
     p.len = ngx_base64_encoded_length(src.len);
 
-    p.data = ngx_palloc(r->pool, p.len);
-    if (p.data == NULL) {
-        return NGX_ERROR;
-    }
+    p.data = lua_newuserdata(L, p.len);
 
     ngx_encode_base64(&p, &src);
 
     lua_pushlstring(L, (char *) p.data, p.len);
 
-    ngx_pfree(r->pool, p.data);
-
     return 1;
 }
 
@@ -570,8 +517,8 @@ ngx_http_lua_ngx_crc32_long(lua_State *L)
 
 
 static int
-ngx_http_lua_ngx_encode_args(lua_State *L) {
-    ngx_http_request_t          *r;
+ngx_http_lua_ngx_encode_args(lua_State *L)
+{
     ngx_str_t                    args;
 
     if (lua_gettop(L) != 1) {
@@ -579,26 +526,16 @@ ngx_http_lua_ngx_encode_args(lua_State *L) {
                           lua_gettop(L));
     }
 
-    lua_pushlightuserdata(L, &ngx_http_lua_request_key);
-    lua_rawget(L, LUA_GLOBALSINDEX);
-    r = lua_touserdata(L, -1);
-    lua_pop(L, 1);
-
     luaL_checktype(L, 1, LUA_TTABLE);
-
-    ngx_http_lua_process_args_option(r, L, 1, &args);
-
+    ngx_http_lua_process_args_option(NULL, L, 1, &args);
     lua_pushlstring(L, (char *) args.data, args.len);
-
-    ngx_pfree(r->pool, args.data);
-
     return 1;
 }
 
 
 static int
-ngx_http_lua_ngx_decode_args(lua_State *L) {
-    ngx_http_request_t          *r;
+ngx_http_lua_ngx_decode_args(lua_State *L)
+{
     u_char                      *buf;
     u_char                      *tmp;
     size_t                       len = 0;
@@ -621,21 +558,12 @@ ngx_http_lua_ngx_decode_args(lua_State *L) {
         max = NGX_HTTP_LUA_MAX_ARGS;
     }
 
-    lua_pushlightuserdata(L, &ngx_http_lua_request_key);
-    lua_rawget(L, LUA_GLOBALSINDEX);
-    r = lua_touserdata(L, -1);
-    lua_pop(L, 1);
-
-    if (r == NULL) {
-        return luaL_error(L, "no request object found");
-    }
-
     tmp = lua_newuserdata(L, len);
     ngx_memcpy(tmp, buf, len);
 
     lua_createtable(L, 0, 4);
 
-    return ngx_http_lua_parse_args(r, L, tmp, tmp + len, max);
+    return ngx_http_lua_parse_args(L, tmp, tmp + len, max);
 }
 
 
@@ -667,4 +595,111 @@ ngx_http_lua_ngx_hmac_sha1(lua_State *L)
 }
 #endif
 
+
+#ifndef NGX_HTTP_LUA_NO_FFI_API
+void
+ngx_http_lua_ffi_md5_bin(const u_char *src, size_t len, u_char *dst)
+{
+    ngx_md5_t     md5;
+
+    ngx_md5_init(&md5);
+    ngx_md5_update(&md5, src, len);
+    ngx_md5_final(dst, &md5);
+}
+
+
+void
+ngx_http_lua_ffi_md5(const u_char *src, size_t len, u_char *dst)
+{
+    ngx_md5_t           md5;
+    u_char              md5_buf[MD5_DIGEST_LENGTH];
+
+    ngx_md5_init(&md5);
+    ngx_md5_update(&md5, src, len);
+    ngx_md5_final(md5_buf, &md5);
+
+    ngx_hex_dump(dst, md5_buf, sizeof(md5_buf));
+}
+
+
+int
+ngx_http_lua_ffi_sha1_bin(const u_char *src, size_t len, u_char *dst)
+{
+#if NGX_HAVE_SHA1
+    ngx_sha1_t               sha;
+
+    ngx_sha1_init(&sha);
+    ngx_sha1_update(&sha, src, len);
+    ngx_sha1_final(dst, &sha);
+
+    return 1;
+#else
+    return 0;
+#endif
+}
+
+
+size_t
+ngx_http_lua_ffi_encode_base64(const u_char *src, size_t slen, u_char *dst)
+{
+    ngx_str_t      in, out;
+
+    in.data = (u_char *) src;
+    in.len = slen;
+
+    out.data = dst;
+
+    ngx_encode_base64(&out, &in);
+
+    return out.len;
+}
+
+
+int
+ngx_http_lua_ffi_decode_base64(const u_char *src, size_t slen, u_char *dst,
+    size_t *dlen)
+{
+    ngx_int_t      rc;
+    ngx_str_t      in, out;
+
+    in.data = (u_char *) src;
+    in.len = slen;
+
+    out.data = dst;
+
+    rc = ngx_decode_base64(&out, &in);
+
+    *dlen = out.len;
+
+    return rc == NGX_OK;
+}
+
+
+size_t
+ngx_http_lua_ffi_unescape_uri(const u_char *src, size_t len, u_char *dst)
+{
+    u_char      *p = dst;
+
+    ngx_http_lua_unescape_uri(&p, (u_char **) &src, len,
+                              NGX_UNESCAPE_URI_COMPONENT);
+    return p - dst;
+}
+
+
+size_t
+ngx_http_lua_ffi_uri_escaped_length(const u_char *src, size_t len)
+{
+    return len + 2 * ngx_http_lua_escape_uri(NULL, (u_char *) src, len,
+                                             NGX_ESCAPE_URI);
+}
+
+
+void
+ngx_http_lua_ffi_escape_uri(const u_char *src, size_t len, u_char *dst)
+{
+    ngx_http_lua_escape_uri(dst, (u_char *) src, len, NGX_ESCAPE_URI);
+}
+
+#endif
+
 /* vi:set ft=c ts=4 sw=4 et fdm=marker: */
